@@ -1,3 +1,5 @@
+// Unicode support by Jim Park -- 08/24/2007
+
 #ifndef __NS_DIALOGS__DEFS_H__
 #define __NS_DIALOGS__DEFS_H__
 
@@ -43,7 +45,7 @@ struct nsControl
 {
   HWND window;
   enum nsControlType type;
-  char userData[USERDATA_SIZE];
+  TCHAR userData[USERDATA_SIZE];
   struct nsControlCallbacks callbacks;
   WNDPROC oldWndProc;
 };
@@ -64,6 +66,6 @@ struct nsDialog
   struct nsControl* controls;
 };
 
-#define NSCONTROL_ID_PROP "NSIS: nsControl pointer property"
+#define NSCONTROL_ID_PROP _T("NSIS: nsControl pointer property")
 
 #endif//__NS_DIALOGS__DEFS_H__
