@@ -1,3 +1,5 @@
+// Unicode support by Jim Park -- 08/10/2007
+
 #include <windows.h>
 
 #include <nsis/pluginapi.h> // nsis plugin
@@ -18,9 +20,20 @@ HINSTANCE g_hInstance;
 struct nsDialog g_dialog;
 extra_parameters* g_pluginParms;
 
+static COLORREF GetLinkColor()
+{
+  COLORREF clr = GetSysColor(COLOR_HOTLIGHT);
+#ifndef _WIN64
+  // COLOR_HOTLIGHT is Win98/2000+. GetSysColorBrush is the correct way to
+  // detect valid colors but here we just assume nobody uses black.
+  if (!clr) clr = RGB(0,0,255);
+#endif
+  return clr;
+}
+
 struct nsControl* NSDFUNC GetControl(HWND hwCtl)
 {
-  unsigned id = (unsigned) GetProp(hwCtl, NSCONTROL_ID_PROP);
+  unsigned id = (unsigned)(UINT_PTR) GetProp(hwCtl, NSCONTROL_ID_PROP);
 
   if (id == 0 || id > g_dialog.controlCount)
   {
@@ -30,9 +43,9 @@ struct nsControl* NSDFUNC GetControl(HWND hwCtl)
   return &g_dialog.controls[id - 1];
 }
 
-BOOL CALLBACK ParentProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
+INT_PTR CALLBACK ParentProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
-  BOOL res;
+  INT_PTR res;
 
   if (message == WM_NOTIFY_OUTER_NEXT)
   {
@@ -74,7 +87,7 @@ LRESULT CALLBACK LinkWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lPar
   return CallWindowProc(ctl->oldWndProc, hwnd, message, wParam, lParam);
 }
 
-BOOL CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
   switch (uMsg)
   {
@@ -91,7 +104,7 @@ BOOL CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
       {
         if (ctl->callbacks.onClick)
         {
-          pushint((int) hwCtl);
+          pushintptr((INT_PTR) hwCtl);
           g_pluginParms->ExecuteCodeSegment(ctl->callbacks.onClick - 1, 0);
         }
       }
@@ -99,34 +112,34 @@ BOOL CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
       {
         if (ctl->callbacks.onChange)
         {
-        pushint((int) hwCtl);
-        g_pluginParms->ExecuteCodeSegment(ctl->callbacks.onChange - 1, 0);
-      }
+          pushintptr((INT_PTR) hwCtl);
+          g_pluginParms->ExecuteCodeSegment(ctl->callbacks.onChange - 1, 0);
+        }
       }
       else if (HIWORD(wParam) == LBN_SELCHANGE && ctl->type == NSCTL_LISTBOX)
       {
         if (ctl->callbacks.onChange)
         {
-        pushint((int) hwCtl);
-        g_pluginParms->ExecuteCodeSegment(ctl->callbacks.onChange - 1, 0);
-      }
+          pushintptr((INT_PTR) hwCtl);
+          g_pluginParms->ExecuteCodeSegment(ctl->callbacks.onChange - 1, 0);
+        }
       }
       else if ((HIWORD(wParam) == CBN_EDITUPDATE || HIWORD(wParam) == CBN_SELCHANGE)
                 && ctl->type == NSCTL_COMBOBOX)
       {
         if (ctl->callbacks.onChange)
         {
-        pushint((int) hwCtl);
-        g_pluginParms->ExecuteCodeSegment(ctl->callbacks.onChange - 1, 0);
-      }
+          pushintptr((INT_PTR) hwCtl);
+          g_pluginParms->ExecuteCodeSegment(ctl->callbacks.onChange - 1, 0);
+        }
       }
       else if (HIWORD(wParam) == STN_CLICKED && ctl->type == NSCTL_STATIC)
       {
         if (ctl->callbacks.onClick)
         {
-        pushint((int) hwCtl);
-        g_pluginParms->ExecuteCodeSegment(ctl->callbacks.onClick - 1, 0);
-      }
+          pushintptr((INT_PTR) hwCtl);
+          g_pluginParms->ExecuteCodeSegment(ctl->callbacks.onClick - 1, 0);
+        }
       }
 
       break;
@@ -143,9 +156,9 @@ BOOL CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
       if (!ctl->callbacks.onNotify)
         break;
 
-      pushint((int) nmhdr);
-      pushint(nmhdr->code);
-      pushint((int) nmhdr->hwndFrom);
+      pushintptr((INT_PTR) nmhdr);
+      pushintptr(nmhdr->code);
+      pushintptr((INT_PTR) nmhdr->hwndFrom);
       g_pluginParms->ExecuteCodeSegment(ctl->callbacks.onNotify - 1, 0);
     }
 
@@ -154,7 +167,7 @@ BOOL CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
     {
       DRAWITEMSTRUCT* lpdis = (DRAWITEMSTRUCT*)lParam;
       RECT rc;
-      char text[1024];
+      TCHAR text[1024];
 
       // http://blogs.msdn.com/oldnewthing/archive/2005/05/03/414317.aspx#414357
       // says we should call SystemParametersInfo(SPI_GETKEYBOARDCUES,...) to make
@@ -171,7 +184,7 @@ BOOL CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
       rc = lpdis->rcItem;
 
       // Get button's text
-      text[0] = '\0';
+      text[0] = _T('\0');
       GetWindowText(lpdis->hwndItem, text, 1024);
 
       // Calculate needed size of the control
@@ -193,9 +206,9 @@ BOOL CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
         if (hideAccel)
           xtraDrawStyle |= DT_HIDEPREFIX;
 
-        // Use blue unless the user has set another using SetCtlColors
-        if (!GetWindowLong(lpdis->hwndItem, GWL_USERDATA))
-          SetTextColor(lpdis->hDC, RGB(0,0,255));
+        // Use the system color unless the user has set another using SetCtlColors
+        if (!GetWindowLongPtr(lpdis->hwndItem, GWLP_USERDATA))
+          SetTextColor(lpdis->hDC, GetLinkColor());
 
         // Draw the text
         DrawText(lpdis->hDC, text, -1, &rc, xtraDrawStyle | DT_CENTER | DT_VCENTER | DT_WORDBREAK);
@@ -242,7 +255,7 @@ static UINT_PTR PluginCallback(enum NSPIM msg)
   return 0;
 }
 
-void __declspec(dllexport) Create(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) Create(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   HWND hwPlacementRect;
   RECT rcPlacement;
@@ -262,7 +275,7 @@ void __declspec(dllexport) Create(HWND hwndParent, int string_size, char *variab
 
   if (g_dialog.hwDialog == NULL)
   {
-    pushstring("error");
+    pushstring(_T("error"));
     return;
   }
 
@@ -276,7 +289,7 @@ void __declspec(dllexport) Create(HWND hwndParent, int string_size, char *variab
     SWP_NOZORDER | SWP_NOACTIVATE
   );
 
-  g_dialog.parentOriginalWndproc = (WNDPROC) SetWindowLong(hwndParent, DWL_DLGPROC, (long) ParentProc);
+  g_dialog.parentOriginalWndproc = (WNDPROC) SetWindowLongPtr(hwndParent, DWLP_DLGPROC, (LONG_PTR) ParentProc);
 
   g_dialog.rtl = FALSE;
 
@@ -285,13 +298,13 @@ void __declspec(dllexport) Create(HWND hwndParent, int string_size, char *variab
 
   g_dialog.callbacks.onBack = 0;
 
-  pushint((int) g_dialog.hwDialog);
+  pushintptr((INT_PTR) g_dialog.hwDialog);
 }
 
-void __declspec(dllexport) CreateControl(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) CreateControl(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
-  char *className;
-  char *text;
+  TCHAR *className;
+  TCHAR *text;
 
   HWND hwItem;
   int x, y, width, height;
@@ -300,18 +313,18 @@ void __declspec(dllexport) CreateControl(HWND hwndParent, int string_size, char
 
   // get info from stack
 
-  className = (char *) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, g_stringsize * 2);
+  className = (TCHAR *) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, (g_stringsize * 2)*sizeof(TCHAR));
   text = &className[g_stringsize];
 
   if (!className)
   {
-    pushstring("error");
+    pushstring(_T("error"));
     return;
   }
 
   if (popstringn(className, 0))
   {
-    pushstring("error");
+    pushstring(_T("error"));
     HeapFree(GetProcessHeap(), 0, className);
     return;
   }
@@ -323,7 +336,7 @@ void __declspec(dllexport) CreateControl(HWND hwndParent, int string_size, char
 
   if (popstringn(text, 0))
   {
-    pushstring("error");
+    pushstring(_T("error"));
     HeapFree(GetProcessHeap(), 0, className);
     return;
   }
@@ -338,21 +351,21 @@ void __declspec(dllexport) CreateControl(HWND hwndParent, int string_size, char
     g_dialog.controls,
     g_dialog.controlCount * sizeof(struct nsControl));
 
-  if (!lstrcmpi(className, "BUTTON"))
+  if (!lstrcmpi(className, _T("BUTTON")))
     g_dialog.controls[id].type = NSCTL_BUTTON;
-  else if (!lstrcmpi(className, "EDIT"))
+  else if (!lstrcmpi(className, _T("EDIT")))
     g_dialog.controls[id].type = NSCTL_EDIT;
-  else if (!lstrcmpi(className, "COMBOBOX"))
+  else if (!lstrcmpi(className, _T("COMBOBOX")))
     g_dialog.controls[id].type = NSCTL_COMBOBOX;
-  else if (!lstrcmpi(className, "LISTBOX"))
+  else if (!lstrcmpi(className, _T("LISTBOX")))
     g_dialog.controls[id].type = NSCTL_LISTBOX;
-  else if (!lstrcmpi(className, "RichEdit"))
+  else if (!lstrcmpi(className, _T("RichEdit")))
     g_dialog.controls[id].type = NSCTL_RICHEDIT;
-  else if (!lstrcmpi(className, "RICHEDIT_CLASS"))
+  else if (!lstrcmpi(className, _T("RICHEDIT_CLASS")))
     g_dialog.controls[id].type = NSCTL_RICHEDIT2;
-  else if (!lstrcmpi(className, "STATIC"))
+  else if (!lstrcmpi(className, _T("STATIC")))
     g_dialog.controls[id].type = NSCTL_STATIC;
-  else if (!lstrcmpi(className, "LINK"))
+  else if (!lstrcmpi(className, _T("LINK")))
     g_dialog.controls[id].type = NSCTL_LINK;
   else
     g_dialog.controls[id].type = NSCTL_UNKNOWN;
@@ -365,7 +378,7 @@ void __declspec(dllexport) CreateControl(HWND hwndParent, int string_size, char
 
   hwItem = CreateWindowEx(
     exStyle,
-    lstrcmpi(className, "LINK") ? className : "BUTTON",
+    lstrcmpi(className, _T("LINK")) ? className : _T("BUTTON"),
     text,
     style,
     x, y, width, height,
@@ -387,11 +400,11 @@ void __declspec(dllexport) CreateControl(HWND hwndParent, int string_size, char
   // set the WndProc for the link control
 
   if(g_dialog.controls[id].type == NSCTL_LINK)
-    g_dialog.controls[id].oldWndProc = (WNDPROC) SetWindowLong(hwItem, GWL_WNDPROC, (long) LinkWndProc);
+    g_dialog.controls[id].oldWndProc = (WNDPROC) SetWindowLongPtr(hwItem, GWLP_WNDPROC, (LONG_PTR) LinkWndProc);
 
   // push back result
 
-  pushint((int) hwItem);
+  pushintptr((INT_PTR) hwItem);
 
   // done
 
@@ -399,23 +412,23 @@ void __declspec(dllexport) CreateControl(HWND hwndParent, int string_size, char
 }
 
 // for backward compatibility (2.29 had CreateItem)
-void __declspec(dllexport) CreateItem(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) CreateItem(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   CreateControl(hwndParent, string_size, variables, stacktop, extra);
 }
 
-void __declspec(dllexport) SetUserData(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) SetUserData(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   HWND hwCtl;
   struct nsControl* ctl;
 
   // get info from stack
 
-  hwCtl = (HWND) popint();
+  hwCtl = (HWND) popintptr();
 
   if (!IsWindow(hwCtl))
   {
-    popint(); // remove user data from stack
+    popintptr(); // remove user data from stack
     return;
   }
 
@@ -431,18 +444,18 @@ void __declspec(dllexport) SetUserData(HWND hwndParent, int string_size, char *v
   popstringn(ctl->userData, USERDATA_SIZE);
 }
 
-void __declspec(dllexport) GetUserData(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) GetUserData(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   HWND hwCtl;
   struct nsControl* ctl;
 
   // get info from stack
 
-  hwCtl = (HWND) popint();
+  hwCtl = (HWND) popintptr();
 
   if (!IsWindow(hwCtl))
   {
-    pushstring("");
+    pushstring(_T(""));
     return;
   }
 
@@ -452,7 +465,7 @@ void __declspec(dllexport) GetUserData(HWND hwndParent, int string_size, char *v
 
   if (ctl == NULL)
   {
-    pushstring("");
+    pushstring(_T(""));
     return;
   }
 
@@ -464,10 +477,10 @@ void __declspec(dllexport) GetUserData(HWND hwndParent, int string_size, char *v
 void CALLBACK TimerProc(HWND hwnd, UINT uMsg, UINT_PTR idEvent, DWORD dwTime)
 {
   // we use a timer proc instead of WM_TIMER to make sure no one messes with the ids but us
-  g_pluginParms->ExecuteCodeSegment(idEvent - 1, 0);
+  g_pluginParms->ExecuteCodeSegment((int)(idEvent - 1), 0);
 }
 
-void __declspec(dllexport) CreateTimer(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) CreateTimer(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   UINT callback;
   UINT interval;
@@ -489,7 +502,7 @@ void __declspec(dllexport) CreateTimer(HWND hwndParent, int string_size, char *v
     TimerProc);
 }
 
-void nsdKillTimer(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void nsdKillTimer(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   UINT id;
 
@@ -511,7 +524,7 @@ void NSDFUNC SetControlCallback(size_t callbackIdx)
 
   // get info from stack
 
-  hwCtl = (HWND) popint();
+  hwCtl = (HWND) popintptr();
   callback = (nsFunction) popint();
 
   if (!IsWindow(hwCtl))
@@ -546,27 +559,27 @@ void NSDFUNC SetDialogCallback(size_t callbackIdx)
 }
 
 
-void __declspec(dllexport) OnClick(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) OnClick(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   SetControlCallback(CTL_CALLBACK_IDX(onClick));
 }
 
-void __declspec(dllexport) OnChange(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) OnChange(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   SetControlCallback(CTL_CALLBACK_IDX(onChange));
 }
 
-void __declspec(dllexport) OnNotify(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) OnNotify(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   SetControlCallback(CTL_CALLBACK_IDX(onNotify));
 }
 
-void __declspec(dllexport) OnBack(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) OnBack(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   SetDialogCallback(DLG_CALLBACK_IDX(onBack));
 }
 
-void __declspec(dllexport) Show(HWND hwndParent, int string_size, char *variables, stack_t **stacktop, extra_parameters *extra)
+void __declspec(dllexport) Show(HWND hwndParent, int string_size, TCHAR *variables, stack_t **stacktop, extra_parameters *extra)
 {
   // tell NSIS to remove old inner dialog and pass handle of the new inner dialog
 
@@ -588,7 +601,7 @@ void __declspec(dllexport) Show(HWND hwndParent, int string_size, char *variable
 
   // reset wndproc
 
-  SetWindowLong(hwndParent, DWL_DLGPROC, (long) g_dialog.parentOriginalWndproc);
+  SetWindowLongPtr(hwndParent, DWLP_DLGPROC, (LONG_PTR) g_dialog.parentOriginalWndproc);
 }
 
 BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
