@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 2002 Amir Szekely <kichik@netvision.net.il>
+ * Copyright (C) 2002-2018 Amir Szekely <kichik@netvision.net.il> and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -12,6 +12,8 @@
  * 
  * This software is provided 'as-is', without any express or implied
  * warranty.
+ *
+ * Reviewed for Unicode support by Jim Park -- 08/21/2007
  */
 
 #if !defined(AFX_DIALOGTEMPLATE_H__C5A973AF_0F56_4BEC_814A_79318E2EB4AC__INCLUDED_)
@@ -22,6 +24,7 @@
 #endif // _MSC_VER > 1000
 
 #include "Platform.h"
+#include "winchar.h"
 
 #include <vector>
 #include <stdexcept>
@@ -43,8 +46,8 @@ struct DialogItemTemplate {
   DWORD  dwStyle;
   WORD   wId;
 
-  WCHAR *szClass;
-  WCHAR *szTitle;
+  WINWCHAR *szClass;
+  WINWCHAR *szTitle;
   char  *szCreationData;
 
   WORD  wCreateDataSize;
@@ -105,7 +108,7 @@ typedef struct {
 
 class CDialogTemplate {
 public:
-  CDialogTemplate(BYTE* pbData, unsigned int uCodePage=CP_ACP);
+  CDialogTemplate(BYTE* pbData, bool build_unicode, unsigned int uCodePage=CP_ACP);
   virtual ~CDialogTemplate();
 
   short GetWidth();
@@ -113,7 +116,7 @@ public:
   DialogItemTemplate* GetItem(WORD wId);
   DialogItemTemplate* GetItemByIdx(DWORD i);
   int   RemoveItem(WORD wId);
-  void  SetFont(char* szFaceName, WORD wFontSize);
+  void  SetFont(TCHAR* szFaceName, WORD wFontSize);
   void  AddItem(DialogItemTemplate item);
 #ifdef _WIN32
   HWND  CreateDummyDialog();
@@ -124,17 +127,19 @@ public:
   void  PixelsToDlgUnits(short& x, short& y);
   void  PixelsToDlgUnits(SIZE& siz);
   void  DlgUnitsToPixels(short& x, short& y);
-  SIZE  GetStringSize(WORD id, char *str);
-  void  RTrimToString(WORD id, char *str, int margins);
-  void  LTrimToString(WORD id, char *str, int margins);
-  void  CTrimToString(WORD id, char *str, int margins);
+  SIZE  GetStringSize(WORD id, TCHAR *str);
+  void  RTrimToString(WORD id, TCHAR *str, int margins);
+  void  LTrimToString(WORD id, TCHAR *str, int margins);
+  void  CTrimToString(WORD id, TCHAR *str, int margins);
 #endif
   void  ConvertToRTL();
   BYTE* Save(DWORD& dwSize);
+  static void FreeSavedTemplate(BYTE*pDT) { delete [] pDT; }
   DWORD GetSize();
 
 private:
   bool  m_bExtended;
+  bool  m_build_unicode;
 
   DWORD m_dwHelpId; // Extended only
 
@@ -145,16 +150,16 @@ private:
   DWORD m_dwExtStyle;
   DWORD m_dwStyle;
 
-  WCHAR *m_szMenu;
-  WCHAR *m_szClass;
-  WCHAR *m_szTitle;
+  WINWCHAR *m_szMenu;
+  WINWCHAR *m_szClass;
+  WINWCHAR *m_szTitle;
 
    // Only if DS_FONT style is set
   short m_sFontSize;
   short m_sFontWeight; // Extended only
   BYTE  m_bItalic; // Extended only
   BYTE  m_bCharset; // Extended only
-  WCHAR *m_szFont;
+  WINWCHAR *m_szFont;
 
   // For (en/de)coding Unicode
   unsigned int m_uCodePage;
