@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2015 Nullsoft and Contributors
+ * Copyright (C) 1999-2018 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -12,6 +12,8 @@
  * 
  * This software is provided 'as-is', without any express or implied
  * warranty.
+ *
+ * Added commands for Unicode support by Jim Park -- 08/21/2007
  */
 
 #ifndef _TOKENS_H_
@@ -57,8 +59,13 @@ enum
   TOK_BRANDINGTEXT,
   TOK_FILEERRORTEXT,
   TOK_INSTPROGRESSFLAGS,
+  TOK_PEDLLCHARACTERISTICS,
+  TOK_PESUBSYSVER,
   TOK_XPSTYLE,
   TOK_REQEXECLEVEL,
+  TOK_MANIFEST_DPIAWARE,
+  TOK_MANIFEST_DPIAWARENESS,
+  TOK_MANIFEST_SUPPORTEDOS,
   TOK_CHANGEUI,
   TOK_ADDBRANDINGIMAGE,
   TOK_SETFONT,
@@ -68,6 +75,12 @@ enum
   TOK_DEFVAR,
   TOK_VI_ADDKEY,
   TOK_VI_SETPRODUCTVERSION,
+  TOK_VI_SETFILEVERSION,
+#ifdef _UNICODE
+  TOK_TARGET,
+  TOK_TARGETCPU,
+  TOK_TARGETUNICODE,
+#endif
 
   TOK_MISCBUTTONTEXT,
   TOK_DETAILSBUTTONTEXT,
@@ -96,8 +109,10 @@ enum
   TOK_P_DEFINE,
   TOK_P_UNDEF,
   TOK_P_PACKEXEHEADER,
+  TOK_P_FINALIZE,
   TOK_P_SYSTEMEXEC,
   TOK_P_EXECUTE,
+  TOK_P_MAKENSIS,
   TOK_P_ADDINCLUDEDIR,
   TOK_P_INCLUDE,
   TOK_P_CD,
@@ -106,9 +121,11 @@ enum
   TOK_P_ERROR,
 
   TOK_P_VERBOSE,
+  TOK_P_PRAGMA,
 
   TOK_P_MACRO,
   TOK_P_MACROEND,
+  TOK_P_MACROUNDEF,
   TOK_P_INSERTMACRO,
   TOK_P_IFMACRODEF,
   TOK_P_IFMACRONDEF,
@@ -116,11 +133,13 @@ enum
   TOK_P_TEMPFILE,
   TOK_P_DELFILE,
   TOK_P_APPENDFILE,
+  TOK_P_GETDLLVERSION,
+  TOK_P_GETTLBVERSION,
 
   TOK_P_SEARCHPARSESTRING,
   TOK_P_SEARCHREPLACESTRING,
 
-  // section/function shit
+  // section/function stuff
   TOK_SECTION,
   TOK_SECTIONEND,
   TOK_SECTIONIN,
@@ -160,6 +179,7 @@ enum
   TOK_EXEC,
   TOK_EXECWAIT,
   TOK_EXECSHELL,
+  TOK_EXECSHELLWAIT,
   TOK_CALLINSTDLL,
   TOK_REGDLL,
   TOK_UNREGDLL,
@@ -170,7 +190,9 @@ enum
   TOK_WRITEREGSTR,
   TOK_WRITEREGEXPANDSTR,
   TOK_WRITEREGBIN,
+  TOK_WRITEREGMULTISZ,
   TOK_WRITEREGDWORD,
+  TOK_WRITEREGNONE,
   TOK_DELETEINISEC,
   TOK_DELETEINISTR,
   TOK_FLUSHINI,
@@ -196,6 +218,7 @@ enum
   TOK_SETERRORS,
   TOK_IFABORT,
   TOK_STRCPY,
+  TOK_UNSAFESTRCPY,
   TOK_STRCMP,
   TOK_STRCMPS,
   TOK_GETTEMPFILENAME,
@@ -215,9 +238,15 @@ enum
   TOK_GETFILETIMELOCAL,
   TOK_STRLEN,
   TOK_INTOP,
+  TOK_INTPTROP,
   TOK_INTCMP,
   TOK_INTCMPU,
+  TOK_INT64CMP,
+  TOK_INT64CMPU,
+  TOK_INTPTRCMP,
+  TOK_INTPTRCMPU,
   TOK_INTFMT,
+  TOK_INT64FMT,
   TOK_ENUMREGKEY,
   TOK_ENUMREGVAL,
   TOK_PUSH,
@@ -236,6 +265,12 @@ enum
   TOK_FILEWRITE,
   TOK_FILEREADBYTE,
   TOK_FILEWRITEBYTE,
+#ifdef _UNICODE
+  TOK_FILEREADUTF16LE,
+  TOK_FILEWRITEUTF16LE,
+  TOK_FILEREADWORD,
+  TOK_FILEWRITEWORD,
+#endif
   TOK_FILESEEK,
   TOK_GETFULLPATHNAME,
   TOK_REBOOT,
